@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2012-2013 DreamWorks Animation LLC
+// Copyright (c) 2012-2014 DreamWorks Animation LLC
 //
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
@@ -33,6 +33,7 @@
 
 #include <openvdb/Types.h>
 #include <openvdb/math/Math.h> // for negative()
+#include "io.h" // for getDataCompression(), etc.
 #include <boost/scoped_array.hpp>
 #include <algorithm>
 #include <iostream>
@@ -53,10 +54,11 @@ namespace io {
 ///     On read, the input stream contains uncompressed data.
 ///
 /// <dt><tt>COMPRESS_ZIP</tt>
-/// <dd>When writing grids other than level sets or fog volumes, apply ZIP compression
-///     to internal and leaf node value buffers.<br>
-///     On read of grids other than level sets or fog volumes, the value buffers
-///     of internal and leaf nodes are ZIP-compressed.
+/// <dd>When writing grids other than level sets or fog volumes, apply
+///     ZLIB compression to internal and leaf node value buffers.<br>
+///     When reading grids other than level sets or fog volumes, indicate that
+///     the value buffers of internal and leaf nodes are ZLIB-compressed.<br>
+///     ZLIB compresses well but is slow.
 ///
 /// <dt><tt>COMPRESS_ACTIVE_MASK</tt>
 /// <dd>When writing a grid of any class, don't output a node's inactive values
@@ -64,11 +66,19 @@ namespace io {
 ///     to permit the lossless reconstruction of inactive values.<br>
 ///     On read, nodes might have been stored without inactive values.
 ///     Where necessary, reconstruct inactive values from available information.
+///
+/// <dt><tt>COMPRESS_BLOSC</tt>
+/// <dd>When writing grids other than level sets or fog volumes, apply
+///     Blosc compression to internal and leaf node value buffers.<br>
+///     When reading grids other than level sets or fog volumes, indicate that
+///     the value buffers of internal and leaf nodes are Blosc-compressed.<br>
+///     Blosc is much faster than ZLIB and produces comparable file sizes.
 /// </dl>
 enum {
     COMPRESS_NONE           = 0,
     COMPRESS_ZIP            = 0x1,
-    COMPRESS_ACTIVE_MASK    = 0x2
+    COMPRESS_ACTIVE_MASK    = 0x2,
+    COMPRESS_BLOSC          = 0x4
 };
 
 /// Return a string describing the given compression flags.
@@ -123,19 +133,30 @@ truncateRealToHalf(const T& val)
 
 OPENVDB_API void zipToStream(std::ostream&, const char* data, size_t numBytes);
 OPENVDB_API void unzipFromStream(std::istream&, char* data, size_t numBytes);
+OPENVDB_API void bloscToStream(std::ostream&, const char* data, size_t valSize, size_t numVals);
+OPENVDB_API void bloscFromStream(std::istream&, char* data, size_t numBytes);
 
 /// @brief Read data from a stream.
-/// @param is          the input stream
-/// @param data        the contiguous array of data to read in
-/// @param count       the number of elements to read in
-/// @param compressed  if @c true, assume the data is ZIP compressed and uncompress it
-/// This default implementation is instantiated only for types whose size
-/// can be determined by the sizeof() operator.
+/// @param is           the input stream
+/// @param data         the contiguous array of data to read in
+/// @param count        the number of elements to read in
+/// @param compression  whether and how the data is compressed (either COMPRESS_NONE,
+///                     COMPRESS_ZIP, COMPRESS_ACTIVE_MASK or COMPRESS_BLOSC)
+/// @throw IoError if @a compression is COMPRESS_BLOSC but OpenVDB was compiled
+/// without Blosc support.
+/// @details This default implementation is instantiated only for types
+/// whose size can be determined by the sizeof() operator.
 template<typename T>
 inline void
-readData(std::istream& is, T* data, Index count, bool compressed)
+readData(std::istream& is, T* data, Index count, uint32_t compression)
 {
-    if (compressed) {
+    if (compression & COMPRESS_BLOSC) {
+#ifdef OPENVDB_USE_BLOSC
+        bloscFromStream(is, reinterpret_cast<char*>(data), sizeof(T) * count);
+#else
+        OPENVDB_THROW(IoError, "Blosc decoding is not supported");
+#endif
+    } else if (compression & COMPRESS_ZIP) {
         unzipFromStream(is, reinterpret_cast<char*>(data), sizeof(T) * count);
     } else {
         is.read(reinterpret_cast<char*>(data), sizeof(T) * count);
@@ -145,7 +166,7 @@ readData(std::istream& is, T* data, Index count, bool compressed)
 /// Specialization for std::string input
 template<>
 inline void
-readData<std::string>(std::istream& is, std::string* data, Index count, bool /*compressed*/)
+readData<std::string>(std::istream& is, std::string* data, Index count, uint32_t /*compression*/)
 {
     for (Index i = 0; i < count; ++i) {
         size_t len = 0;
@@ -167,18 +188,18 @@ template<bool IsReal, typename T> struct HalfReader;
 /// Partial specialization for non-floating-point types (no half to float promotion)
 template<typename T>
 struct HalfReader</*IsReal=*/false, T> {
-    static inline void read(std::istream& is, T* data, Index count, bool compressed) {
-        readData(is, data, count, compressed);
+    static inline void read(std::istream& is, T* data, Index count, uint32_t compression) {
+        readData(is, data, count, compression);
     }
 };
 /// Partial specialization for floating-point types
 template<typename T>
 struct HalfReader</*IsReal=*/true, T> {
     typedef typename RealToHalf<T>::HalfT HalfT;
-    static inline void read(std::istream& is, T* data, Index count, bool compressed) {
+    static inline void read(std::istream& is, T* data, Index count, uint32_t compression) {
         if (count < 1) return;
         std::vector<HalfT> halfData(count); // temp buffer into which to read half float values
-        readData<HalfT>(is, reinterpret_cast<HalfT*>(&halfData[0]), count, compressed);
+        readData<HalfT>(is, reinterpret_cast<HalfT*>(&halfData[0]), count, compression);
         // Copy half float values from the temporary buffer to the full float output array.
         std::copy(halfData.begin(), halfData.end(), data);
     }
@@ -186,17 +207,26 @@ struct HalfReader</*IsReal=*/true, T> {
 
 
 /// Write data to a stream.
-/// @param os        the output stream
-/// @param data      the contiguous array of data to write
-/// @param count     the number of elements to write out
-/// @param compress  if @c true, apply ZIP compression to the data
-/// This default implementation is instantiated only for types whose size
-/// can be determined by the sizeof() operator.
+/// @param os           the output stream
+/// @param data         the contiguous array of data to write
+/// @param count        the number of elements to write out
+/// @param compression  whether and how to compress the data (either COMPRESS_NONE,
+///                     COMPRESS_ZIP, COMPRESS_ACTIVE_MASK or COMPRESS_BLOSC)
+/// @throw IoError if @a compression is COMPRESS_BLOSC but OpenVDB was compiled
+/// without Blosc support.
+/// @details This default implementation is instantiated only for types
+/// whose size can be determined by the sizeof() operator.
 template<typename T>
 inline void
-writeData(std::ostream &os, const T *data, Index count, bool compress)
+writeData(std::ostream &os, const T *data, Index count, uint32_t compression)
 {
-    if (compress) {
+    if (compression & COMPRESS_BLOSC) {
+#ifdef OPENVDB_USE_BLOSC
+        bloscToStream(os, reinterpret_cast<const char*>(data), sizeof(T), count);
+#else
+        OPENVDB_THROW(IoError, "Blosc encoding is not supported");
+#endif
+    } else if (compression & COMPRESS_ZIP) {
         zipToStream(os, reinterpret_cast<const char*>(data), sizeof(T) * count);
     } else {
         os.write(reinterpret_cast<const char*>(data), sizeof(T) * count);
@@ -204,10 +234,10 @@ writeData(std::ostream &os, const T *data, Index count, bool compress)
 }
 
 /// Specialization for std::string output
-/// @todo Add compression
 template<>
 inline void
-writeData<std::string>(std::ostream& os, const std::string* data, Index count, bool /*compress*/)
+writeData<std::string>(std::ostream& os, const std::string* data, Index count,
+    uint32_t /*compression*/) ///< @todo add compression
 {
     for (Index i = 0; i < count; ++i) {
         const size_t len = data[i].size();
@@ -225,20 +255,20 @@ template<bool IsReal, typename T> struct HalfWriter;
 /// Partial specialization for non-floating-point types (no float to half quantization)
 template<typename T>
 struct HalfWriter</*IsReal=*/false, T> {
-    static inline void write(std::ostream& os, const T* data, Index count, bool compress) {
-        writeData(os, data, count, compress);
+    static inline void write(std::ostream& os, const T* data, Index count, uint32_t compression) {
+        writeData(os, data, count, compression);
     }
 };
 /// Partial specialization for floating-point types
 template<typename T>
 struct HalfWriter</*IsReal=*/true, T> {
     typedef typename RealToHalf<T>::HalfT HalfT;
-    static inline void write(std::ostream& os, const T* data, Index count, bool compress) {
+    static inline void write(std::ostream& os, const T* data, Index count, uint32_t compression) {
         if (count < 1) return;
         // Convert full float values to half float, then output the half float array.
         std::vector<HalfT> halfData(count);
-        std::copy(data, data + count, halfData.begin());
-        writeData<HalfT>(os, reinterpret_cast<const HalfT*>(&halfData[0]), count, compress);
+        for (Index i = 0; i < count; ++i) halfData[i] = static_cast<HalfT>(data[i]);
+        writeData<HalfT>(os, reinterpret_cast<const HalfT*>(&halfData[0]), count, compression);
     }
 };
 #ifdef _MSC_VER
@@ -246,12 +276,14 @@ struct HalfWriter</*IsReal=*/true, T> {
 template<>
 struct HalfWriter</*IsReal=*/true, double> {
     typedef RealToHalf<double>::HalfT HalfT;
-    static inline void write(std::ostream& os, const double* data, Index count, bool compress) {
+    static inline void write(std::ostream& os, const double* data, Index count,
+        uint32_t compression)
+    {
         if (count < 1) return;
         // Convert full float values to half float, then output the half float array.
         std::vector<HalfT> halfData(count);
         for (Index i = 0; i < count; ++i) halfData[i] = float(data[i]);
-        writeData<HalfT>(os, reinterpret_cast<const HalfT*>(&halfData[0]), count, compress);
+        writeData<HalfT>(os, reinterpret_cast<const HalfT*>(&halfData[0]), count, compression);
     }
 };
 #endif // _MSC_VER
@@ -279,9 +311,7 @@ readCompressedValues(std::istream& is, ValueT* destBuf, Index destCount,
 {
     // Get the stream's compression settings.
     const uint32_t compression = getDataCompression(is);
-    const bool
-        zipped = compression & COMPRESS_ZIP,
-        maskCompressed = compression & COMPRESS_ACTIVE_MASK;
+    const bool maskCompressed = compression & COMPRESS_ACTIVE_MASK;
 
     int8_t metadata = NO_MASK_AND_ALL_VALS;
     if (getFormatVersion(is) >= OPENVDB_FILE_VERSION_NODE_MASK_COMPRESSION) {
@@ -338,9 +368,9 @@ readCompressedValues(std::istream& is, ValueT* destBuf, Index destCount,
 
     // Read in the buffer.
     if (fromHalf) {
-        HalfReader<RealToHalf<ValueT>::isReal, ValueT>::read(is, tempBuf, tempCount, zipped);
+        HalfReader<RealToHalf<ValueT>::isReal, ValueT>::read(is, tempBuf, tempCount, compression);
     } else {
-        readData<ValueT>(is, tempBuf, tempCount, zipped);
+        readData<ValueT>(is, tempBuf, tempCount, compression);
     }
 
     // If mask compression is enabled and the number of active values read into
@@ -390,9 +420,7 @@ writeCompressedValues(std::ostream& os, ValueT* srcBuf, Index srcCount,
 
     // Get the stream's compression settings.
     const uint32_t compress = getDataCompression(os);
-    const bool
-        zip = compress & COMPRESS_ZIP,
-        maskCompress = compress & COMPRESS_ACTIVE_MASK;
+    const bool maskCompress = compress & COMPRESS_ACTIVE_MASK;
 
     Index tempCount = srcCount;
     ValueT* tempBuf = srcBuf;
@@ -500,7 +528,7 @@ writeCompressedValues(std::ostream& os, ValueT* srcBuf, Index srcCount,
                 }
             } else {
                 // Write one of at most two distinct inactive values.
-                ValueT truncatedVal = truncateRealToHalf(inactiveVal[0]);
+                ValueT truncatedVal = static_cast<ValueT>(truncateRealToHalf(inactiveVal[0]));
                 os.write(reinterpret_cast<const char*>(&truncatedVal), sizeof(ValueT));
                 if (metadata == MASK_AND_TWO_INACTIVE_VALS) {
                     // Write the second of two distinct inactive values.
@@ -554,9 +582,9 @@ writeCompressedValues(std::ostream& os, ValueT* srcBuf, Index srcCount,
 
     // Write out the buffer.
     if (toHalf) {
-        HalfWriter<RealToHalf<ValueT>::isReal, ValueT>::write(os, tempBuf, tempCount, zip);
+        HalfWriter<RealToHalf<ValueT>::isReal, ValueT>::write(os, tempBuf, tempCount, compress);
     } else {
-        writeData(os, tempBuf, tempCount, zip);
+        writeData(os, tempBuf, tempCount, compress);
     }
 }
 
@@ -566,6 +594,6 @@ writeCompressedValues(std::ostream& os, ValueT* srcBuf, Index srcCount,
 
 #endif // OPENVDB_IO_COMPRESSION_HAS_BEEN_INCLUDED
 
-// Copyright (c) 2012-2013 DreamWorks Animation LLC
+// Copyright (c) 2012-2014 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
