@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2012-2013 DreamWorks Animation LLC
+// Copyright (c) 2012-2014 DreamWorks Animation LLC
 //
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
@@ -98,7 +98,7 @@ Quat<T> slerp(const Quat<T> &q1, const Quat<T> &q2, T t, T tolerance=0.00001)
                        a * q1[2] + b * q2[2], a * q1[3] + b * q2[3]);
     }
 
-} 
+}
 
 template<typename T>
 class Quat
@@ -133,32 +133,32 @@ public:
     {
         // assert( REL_EQ(axis.length(), 1.) );
 
-        T s = sin(angle*T(0.5));
+        T s = T(sin(angle*T(0.5)));
 
         mm[0] = axis.x() * s;
         mm[1] = axis.y() * s;
         mm[2] = axis.z() * s;
 
-        mm[3] = cos(angle*T(0.5));
+        mm[3] = T(cos(angle*T(0.5)));
 
     }
 
     /// Constructor given rotation as axis and angle
     Quat(math::Axis axis, T angle)
     {
-        T s = sin(angle*T(0.5));
+        T s = T(sin(angle*T(0.5)));
 
         mm[0] = (axis==math::X_AXIS) * s;
         mm[1] = (axis==math::Y_AXIS) * s;
         mm[2] = (axis==math::Z_AXIS) * s;
 
-        mm[3] = cos(angle*T(0.5));
+        mm[3] = T(cos(angle*T(0.5)));
     }
 
     /// Constructor given a rotation matrix
     template<typename T1>
-    Quat(const Mat3<T1> &rot) { 
-       
+    Quat(const Mat3<T1> &rot) {
+
         // verify that the matrix is really a rotation
         if(!isUnitary(rot)) {  // unitary is reflection or rotation
              OPENVDB_THROW(ArithmeticError,
@@ -171,7 +171,7 @@ public:
 
         T trace = (T)rot.trace();
         if (trace > 0) {
-            
+
             T q_w = 0.5 * std::sqrt(trace+1);
             T factor = 0.25 / q_w;
 
@@ -257,7 +257,7 @@ public:
 
         if ( sqrLength > 1.0e-8 ) {
 
-            return T(2.0) * acos(mm[3]);
+            return T(T(2.0) * acos(mm[3]));
 
         } else {
 
@@ -272,7 +272,7 @@ public:
 
         if ( sqrLength > 1.0e-8 ) {
 
-            T invLength = T(1)/sqrt(sqrLength);
+            T invLength = T(T(1)/sqrt(sqrLength));
 
             return Vec3<T>( mm[0]*invLength, mm[1]*invLength, mm[2]*invLength );
         } else {
@@ -296,14 +296,14 @@ public:
     /// the axis must be unit vector
     Quat& setAxisAngle(const Vec3<T>& axis, T angle)
     {
-     
-        T s = sin(angle*T(0.5));
+
+        T s = T(sin(angle*T(0.5)));
 
         mm[0] = axis.x() * s;
         mm[1] = axis.y() * s;
         mm[2] = axis.z() * s;
 
-        mm[3] = cos(angle*T(0.5));
+        mm[3] = T(cos(angle*T(0.5)));
 
         return *this;
     } // axisAngleTest
@@ -446,7 +446,7 @@ public:
         mm[3] = q1.mm[3] + q2.mm[3];
 
         return *this;
-    } 
+    }
 
     /// this = q1 - q2
     /// "this", q1 and q2 need not be distinct objects, e.g. q.sub(q1,q);
@@ -458,7 +458,7 @@ public:
         mm[3] = q1.mm[3] - q2.mm[3];
 
         return *this;
-    } 
+    }
 
     /// this = q1 * q2
     /// q1 and q2 must be distinct objects than "this", e.g.  q.mult(q1,q2);
@@ -474,7 +474,7 @@ public:
                 q1.mm[1]*q2.mm[1] - q1.mm[2]*q2.mm[2];
 
         return *this;
-    } 
+    }
 
     /// this =  scalar*q, q need not be distinct object than "this",
     /// e.g. q.scale(1.5,q1);
@@ -487,7 +487,7 @@ public:
 
         return *this;
     }
-    
+
     /// Dot product
     T dot(const Quat &q) const
     {
@@ -502,16 +502,16 @@ public:
                         +z()*omega.x() +w()*omega.y() -x()*omega.z() ,
                         -y()*omega.x() +x()*omega.y() +w()*omega.z() ,
                         -x()*omega.x() -y()*omega.y() -z()*omega.z() );
-    } 
+    }
 
     /// this = normalized this
-    bool normalize(T eps =1.0e-8)
+    bool normalize(T eps = T(1.0e-8))
     {
-        T d = sqrt(mm[0]*mm[0] + mm[1]*mm[1] + mm[2]*mm[2] + mm[3]*mm[3]);
+        T d = T(sqrt(mm[0]*mm[0] + mm[1]*mm[1] + mm[2]*mm[2] + mm[3]*mm[3]));
         if( isApproxEqual(d, T(0.0), eps) ) return false;
         *this *= ( T(1)/d );
         return true;
-    } 
+    }
 
     /// this = normalized this
     Quat unit() const
@@ -521,10 +521,10 @@ public:
             OPENVDB_THROW(ArithmeticError,
                 "Normalizing degenerate quaternion");
         return *this / d;
-    } 
+    }
 
     /// returns inverse of this
-    Quat inverse(T tolerance = 0)
+    Quat inverse(T tolerance = T(0))
     {
         T d = mm[0]*mm[0] + mm[1]*mm[1] + mm[2]*mm[2] + mm[3]*mm[3];
         if( isApproxEqual(d, T(0.0), tolerance) )
@@ -533,7 +533,7 @@ public:
         Quat result = *this/-d;
         result.mm[3] = -result.mm[3];
         return result;
-    } 
+    }
 
 
     /// Return the conjugate of "this", same as invert without
@@ -541,14 +541,14 @@ public:
     Quat conjugate() const
     {
         return Quat<T>(-mm[0], -mm[1], -mm[2], mm[3]);
-    } 
+    }
 
     /// Return rotated vector by "this" quaternion
     Vec3<T> rotateVector(const Vec3<T> &v) const
     {
         Mat3<T> m(*this);
         return m.transform(v);
-    } 
+    }
 
     /// Predefined constants, e.g.   Quat q = Quat::identity();
     static Quat zero() { return Quat<T>(0,0,0,0); }
@@ -598,7 +598,7 @@ template <typename S, typename T>
 Quat<T> operator*(S scalar, const Quat<T> &q) { return q*scalar; }
 
 
-/// @brief Interpolate between m1 and m2. 
+/// @brief Interpolate between m1 and m2.
 /// Converts to quaternion  form and uses slerp
 /// m1 and m2 must be rotation matrices!
 template <typename T, typename T0>
@@ -608,9 +608,9 @@ Mat3<T> slerp(const Mat3<T0> &m1, const Mat3<T0> &m2, T t)
 
     Quat<T> q1(m1);
     Quat<T> q2(m2);
-   
+
     if (q1.dot(q2) < 0) q2 *= -1;
-    
+
     Quat<T> qslerp = slerp<T>(q1, q2, static_cast<T>(t));
     MatType m = rotation<MatType>(qslerp);
     return m;
@@ -632,11 +632,11 @@ Mat3<T> bezLerp(const Mat3<T0> &m1, const Mat3<T0> &m2,
                 T t)
 {
     Mat3<T> m00, m01, m02, m10, m11;
-    
+
     m00 = slerp(m1, m2, t);
     m01 = slerp(m2, m3, t);
     m02 = slerp(m3, m4, t);
-    
+
     m10 = slerp(m00, m01, t);
     m11 = slerp(m01, m02, t);
 
@@ -653,6 +653,6 @@ typedef Quat<double> Quatd;
 #endif //OPENVDB_MATH_QUAT_H_HAS_BEEN_INCLUDED
 
 // ---------------------------------------------------------------------------
-// Copyright (c) 2012-2013 DreamWorks Animation LLC
+// Copyright (c) 2012-2014 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
