@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2012-2013 DreamWorks Animation LLC
+// Copyright (c) 2012-2014 DreamWorks Animation LLC
 //
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
@@ -46,7 +46,7 @@ template <>
 inline Index32
 StringMetadata::size() const
 {
-    return mValue.size();
+    return Index32(mValue.size());
 }
 
 
@@ -70,6 +70,6 @@ StringMetadata::writeValue(std::ostream &os) const
 
 #endif // OPENVDB_METADATA_STRINGMETADATA_HAS_BEEN_INCLUDED
 
-// Copyright (c) 2012-2013 DreamWorks Animation LLC
+// Copyright (c) 2012-2014 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
