@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2012-2013 DreamWorks Animation LLC
+// Copyright (c) 2012-2014 DreamWorks Animation LLC
 //
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
@@ -110,12 +110,16 @@ class LeafManager
 {
 public:
     typedef TreeT                                                      TreeType;
+    typedef typename TreeT::ValueType                                  ValueType;
+    typedef typename TreeT::RootNodeType                               RootNodeType;
     typedef typename TreeType::LeafNodeType                            NonConstLeafType;
     typedef typename CopyConstness<TreeType, NonConstLeafType>::Type   LeafType;
+    typedef LeafType                                                   LeafNodeType;
     typedef typename leafmgr::TreeTraits<TreeT>::LeafIterType          LeafIterType;
     typedef typename LeafType::Buffer                                  NonConstBufferType;
     typedef typename CopyConstness<TreeType, NonConstBufferType>::Type BufferType;
     typedef tbb::blocked_range<size_t>                                 RangeType;//leaf index range
+    static const Index DEPTH = 2;//root + leafs
 
     static const bool IsConstTree = leafmgr::TreeTraits<TreeT>::IsConstTree;
 
@@ -294,9 +298,18 @@ public:
     /// Return the number of leaf nodes.
     size_t leafCount() const { return mLeafCount; }
 
-    /// Return the tree associated with this manager.
+    /// Return a const reference to tree associated with this manager.
+    const TreeType& tree() const { return *mTree; }
+
+    /// Return a reference to the tree associated with this manager.
     TreeType& tree() { return *mTree; }
 
+    /// Return a const reference to root node associated with this manager.
+    const RootNodeType& root() const { return mTree->root(); }
+
+    /// Return a reference to the root node associated with this manager.
+    RootNodeType& root() { return mTree->root(); }
+
     /// Return @c true if the tree associated with this manager is immutable.
     bool isConstTree() const { return this->IsConstTree; }
 
@@ -466,6 +479,42 @@ public:
         transform.run(this->leafRange(grainSize), threaded);
     }
 
+
+    template<typename ArrayT>
+    void getNodes(ArrayT& array)
+    {
+        typedef typename ArrayT::value_type T;
+        BOOST_STATIC_ASSERT(boost::is_pointer<T>::value);
+        typedef typename boost::mpl::if_<boost::is_const<typename boost::remove_pointer<T>::type>,
+            const LeafType, LeafType>::type LeafT;
+
+        OPENVDB_NO_UNREACHABLE_CODE_WARNING_BEGIN
+        if (boost::is_same<T, LeafT*>::value) {
+            array.resize(mLeafCount);
+            for (size_t i=0; i<mLeafCount; ++i) array[i] = reinterpret_cast<T>(mLeafs[i]);
+        } else {
+            mTree->getNodes(array);
+        }
+        OPENVDB_NO_UNREACHABLE_CODE_WARNING_END
+            }
+
+    template<typename ArrayT>
+    void getNodes(ArrayT& array) const
+    {
+        typedef typename ArrayT::value_type T;
+        BOOST_STATIC_ASSERT(boost::is_pointer<T>::value);
+        BOOST_STATIC_ASSERT(boost::is_const<typename boost::remove_pointer<T>::type>::value);
+
+        OPENVDB_NO_UNREACHABLE_CODE_WARNING_BEGIN
+        if (boost::is_same<T, const LeafType*>::value) {
+            array.resize(mLeafCount);
+            for (size_t i=0; i<mLeafCount; ++i) array[i] = reinterpret_cast<T>(mLeafs[i]);
+        } else {
+            mTree->getNodes(array);
+        }
+        OPENVDB_NO_UNREACHABLE_CODE_WARNING_END
+    }
+
     ////////////////////////////////////////////////////////////////////////////////////
     // All methods below are for internal use only and should never be called directly
 
@@ -478,7 +527,18 @@ public:
 
 
 
-private:
+  private:
+
+    // This a simple wrapper for a c-style array so it mimics the api
+    // of a std container, e.g. std::vector or std::deque, and can be
+    // passed to Tree::getNodes().
+    struct MyArray {
+        typedef LeafType* value_type;//required by Tree::getNodes
+        value_type* ptr;
+        MyArray(value_type* array) : ptr(array) {}
+        void push_back(value_type leaf) { *ptr++ = leaf; }//required by Tree::getNodes
+    };
+
     void initLeafArray()
     {
         const size_t leafCount = mTree->leafCount();
@@ -487,8 +547,8 @@ private:
             mLeafs = (leafCount == 0) ? NULL : new LeafType*[leafCount];
             mLeafCount = leafCount;
         }
-        LeafIterType iter = mTree->beginLeaf();
-        for (size_t n = 0; n != leafCount; ++n, ++iter) mLeafs[n] = iter.getLeaf();
+        MyArray a(mLeafs);
+        mTree->getNodes(a);
     }
 
     void initAuxBuffers(bool serial)
@@ -605,6 +665,6 @@ struct LeafManagerImpl<LeafManager<const TreeT> >
 
 #endif // OPENVDB_TREE_LEAFMANAGER_HAS_BEEN_INCLUDED
 
-// Copyright (c) 2012-2013 DreamWorks Animation LLC
+// Copyright (c) 2012-2014 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
