@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2012-2013 DreamWorks Animation LLC
+// Copyright (c) 2012-2014 DreamWorks Animation LLC
 //
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
@@ -64,7 +64,9 @@ TestInternalOrigin::test()
     typedef openvdb::tree::Tree4<float,5,4,3>::Type FloatTree4;
     FloatTree4 tree(0.0f);
     std::set<openvdb::Coord>::iterator iter=indices.begin();
-    for (int n=0; iter!=indices.end(); ++n, ++iter) tree.setValue(*iter,1+n*0.5f);
+    for (int n = 0; iter != indices.end(); ++n, ++iter) {
+        tree.setValue(*iter, float(1.0 + float(n) * 0.5f));
+    }
 
     openvdb::Coord C3, G;
     typedef FloatTree4::RootNodeType Node0;
@@ -99,6 +101,6 @@ TestInternalOrigin::test()
     CPPUNIT_ASSERT(indices.size() == 0);
 }
 
-// Copyright (c) 2012-2013 DreamWorks Animation LLC
+// Copyright (c) 2012-2014 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
