@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2012-2013 DreamWorks Animation LLC
+// Copyright (c) 2012-2014 DreamWorks Animation LLC
 //
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
@@ -42,13 +42,19 @@ class TestStats: public CppUnit::TestCase
 {
 public:
     CPPUNIT_TEST_SUITE(TestStats);
+    CPPUNIT_TEST(testExtrema);
     CPPUNIT_TEST(testStats);
+    CPPUNIT_TEST(testHistogram);
+    CPPUNIT_TEST(testGridExtrema);
     CPPUNIT_TEST(testGridStats);
     CPPUNIT_TEST(testGridHistogram);
     CPPUNIT_TEST(testGridOperatorStats);
     CPPUNIT_TEST_SUITE_END();
 
+    void testExtrema();
     void testStats();
+    void testHistogram();
+    void testGridExtrema();
     void testGridStats();
     void testGridHistogram();
     void testGridOperatorStats();
@@ -56,6 +62,81 @@ public:
 
 CPPUNIT_TEST_SUITE_REGISTRATION(TestStats);
 
+void
+TestStats::testExtrema()
+{
+    {// trivial test
+        openvdb::math::Extrema s;
+        s.add(0);
+        s.add(1);
+        CPPUNIT_ASSERT_EQUAL(2, int(s.size()));
+        CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, s.min(), 0.000001);
+        CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, s.max(), 0.000001);
+        //s.print("test");
+    }
+    {// non-trivial test
+        openvdb::math::Extrema s;
+        const int data[5]={600, 470, 170, 430, 300};
+        for (int i=0; i<5; ++i) s.add(data[i]);
+        CPPUNIT_ASSERT_EQUAL(5, int(s.size()));
+        CPPUNIT_ASSERT_DOUBLES_EQUAL(data[2], s.min(), 0.000001);
+        CPPUNIT_ASSERT_DOUBLES_EQUAL(data[0], s.max(), 0.000001);
+        //s.print("test");
+    }
+    {// non-trivial test of Extrema::add(Extrema)
+        openvdb::math::Extrema s, t;
+        const int data[5]={600, 470, 170, 430, 300};
+        for (int i=0; i<3; ++i) s.add(data[i]);
+        for (int i=3; i<5; ++i) t.add(data[i]);
+        s.add(t);
+        CPPUNIT_ASSERT_EQUAL(5, int(s.size()));
+        CPPUNIT_ASSERT_DOUBLES_EQUAL(data[2], s.min(), 0.000001);
+        CPPUNIT_ASSERT_DOUBLES_EQUAL(data[0], s.max(), 0.000001);
+        //s.print("test");
+    }
+    {// Trivial test of Extrema::add(value, n)
+        openvdb::math::Extrema s;
+        const double val = 3.45;
+        const uint64_t n = 57;
+        s.add(val, 57);
+        CPPUNIT_ASSERT_EQUAL(n, s.size());
+        CPPUNIT_ASSERT_DOUBLES_EQUAL(val, s.min(), 0.000001);
+        CPPUNIT_ASSERT_DOUBLES_EQUAL(val, s.max(), 0.000001);
+    }
+    {// Test 1 of Extrema::add(value), Extrema::add(value, n) and Extrema::add(Extrema)
+        openvdb::math::Extrema s, t;
+        const double val1 = 1.0, val2 = 3.0;
+        const uint64_t n1 = 1, n2 =1;
+        s.add(val1,  n1);
+        CPPUNIT_ASSERT_EQUAL(uint64_t(n1), s.size());
+        CPPUNIT_ASSERT_DOUBLES_EQUAL(val1, s.min(),      0.000001);
+        CPPUNIT_ASSERT_DOUBLES_EQUAL(val1, s.max(),      0.000001);
+        for (uint64_t i=0; i<n2; ++i) t.add(val2);
+        s.add(t);
+        CPPUNIT_ASSERT_EQUAL(uint64_t(n2), t.size());
+        CPPUNIT_ASSERT_DOUBLES_EQUAL(val2, t.min(),      0.000001);
+        CPPUNIT_ASSERT_DOUBLES_EQUAL(val2, t.max(),      0.000001);
+
+        CPPUNIT_ASSERT_EQUAL(uint64_t(n1+n2), s.size());
+        CPPUNIT_ASSERT_DOUBLES_EQUAL(val1,    s.min(),  0.000001);
+        CPPUNIT_ASSERT_DOUBLES_EQUAL(val2,    s.max(),  0.000001);
+    }
+    {// Non-trivial test of Extrema::add(value, n)
+        openvdb::math::Extrema s;
+        s.add(3.45,  6);
+        s.add(1.39,  2);
+        s.add(2.56, 13);
+        s.add(0.03);
+        openvdb::math::Extrema t;
+        for (int i=0; i< 6; ++i) t.add(3.45);
+        for (int i=0; i< 2; ++i) t.add(1.39);
+        for (int i=0; i<13; ++i) t.add(2.56);
+        t.add(0.03);
+        CPPUNIT_ASSERT_EQUAL(s.size(), t.size());
+        CPPUNIT_ASSERT_DOUBLES_EQUAL(s.min(), t.min(),  0.000001);
+        CPPUNIT_ASSERT_DOUBLES_EQUAL(s.max(), t.max(),  0.000001);
+    }
+}
 
 void
 TestStats::testStats()
@@ -216,7 +297,14 @@ TestStats::testStats()
         CPPUNIT_ASSERT_DOUBLES_EQUAL(s.mean(),t.mean(), 0.000001);
         CPPUNIT_ASSERT_DOUBLES_EQUAL(s.variance(), t.variance(), 0.000001);
     }
-    {// Histogram test
+
+    //std::cerr << "\nCompleted TestStats::testStats!\n" << std::endl;
+}
+
+void
+TestStats::testHistogram()
+{
+     {// Histogram test
         openvdb::math::Stats s;
         const int data[5]={600, 470, 170, 430, 300};
         for (int i=0; i<5; ++i) s.add(data[i]);
@@ -238,10 +326,8 @@ TestStats::testStats()
         for (int i=0; i<N; ++i) CPPUNIT_ASSERT(h.add(N/2-i));
         //h.print("print-test");
     }
-    //std::cerr << "\nCompleted TestStats::testStats!\n" << std::endl;
 }
 
-
 namespace {
 
 struct GradOp
@@ -252,7 +338,8 @@ struct GradOp
 
     GradOp(const GridT& grid): acc(grid.getConstAccessor()) {}
 
-    void operator()(const GridT::ValueOnCIter& it, openvdb::math::Stats& stats) const
+    template <typename StatsT>
+    void operator()(const GridT::ValueOnCIter& it, StatsT& stats) const
     {
         typedef openvdb::math::ISGradient<openvdb::math::FD_1ST> GradT;
         if (it.isVoxelValue()) {
@@ -274,6 +361,102 @@ struct GradOp
 
 } // unnamed namespace
 
+void
+TestStats::testGridExtrema()
+{
+    using namespace openvdb;
+
+    const int DIM = 109;
+    {
+        const float background = 0.0;
+        FloatGrid grid(background);
+        {
+            // Compute active value statistics for a grid with a single active voxel.
+            grid.tree().setValue(Coord(0), /*value=*/42.0);
+            math::Extrema ex = tools::extrema(grid.cbeginValueOn());
+
+            CPPUNIT_ASSERT_DOUBLES_EQUAL(42.0, ex.min(),  /*tolerance=*/0.0);
+            CPPUNIT_ASSERT_DOUBLES_EQUAL(42.0, ex.max(),  /*tolerance=*/0.0);
+
+            // Compute inactive value statistics for a grid with only background voxels.
+            grid.tree().setValueOff(Coord(0), background);
+            ex = tools::extrema(grid.cbeginValueOff());
+
+            CPPUNIT_ASSERT_DOUBLES_EQUAL(background, ex.min(),  /*tolerance=*/0.0);
+            CPPUNIT_ASSERT_DOUBLES_EQUAL(background, ex.max(),  /*tolerance=*/0.0);
+        }
+
+        // Compute active value statistics for a grid with two active voxel populations
+        // of the same size but two different values.
+        grid.fill(CoordBBox::createCube(Coord(0), DIM), /*value=*/1.0);
+        grid.fill(CoordBBox::createCube(Coord(-300), DIM), /*value=*/-3.0);
+
+        CPPUNIT_ASSERT_EQUAL(Index64(2 * DIM * DIM * DIM), grid.activeVoxelCount());
+
+        for (int threaded = 0; threaded <= 1; ++threaded) {
+            math::Extrema ex = tools::extrema(grid.cbeginValueOn(), threaded);
+
+            CPPUNIT_ASSERT_DOUBLES_EQUAL(double(-3.0), ex.min(),  /*tolerance=*/0.0);
+            CPPUNIT_ASSERT_DOUBLES_EQUAL(double(1.0),  ex.max(),  /*tolerance=*/0.0);
+        }
+
+        // Compute active value statistics for just the positive values.
+        for (int threaded = 0; threaded <= 1; ++threaded) {
+            struct Local {
+                static void addIfPositive(const FloatGrid::ValueOnCIter& it, math::Extrema& ex)
+                {
+                    const float f = *it;
+                    if (f > 0.0) {
+                        if (it.isVoxelValue()) ex.add(f);
+                        else ex.add(f, it.getVoxelCount());
+                    }
+                }
+            };
+            math::Extrema ex =
+                tools::extrema(grid.cbeginValueOn(), &Local::addIfPositive, threaded);
+
+            CPPUNIT_ASSERT_DOUBLES_EQUAL(double(1.0), ex.min(),  /*tolerance=*/0.0);
+            CPPUNIT_ASSERT_DOUBLES_EQUAL(double(1.0), ex.max(),  /*tolerance=*/0.0);
+        }
+
+        // Compute active value statistics for the first-order gradient.
+        for (int threaded = 0; threaded <= 1; ++threaded) {
+            // First, using a custom ValueOp...
+            math::Extrema ex = tools::extrema(grid.cbeginValueOn(), GradOp(grid), threaded);
+            CPPUNIT_ASSERT_DOUBLES_EQUAL(double(0.0), ex.min(), /*tolerance=*/0.0);
+            CPPUNIT_ASSERT_DOUBLES_EQUAL(
+                double(9.0 + 9.0 + 9.0), ex.max() * ex.max(), /*tol=*/1.0e-3);
+                // max gradient is (dx, dy, dz) = (-3 - 0, -3 - 0, -3 - 0)
+
+            // ...then using tools::opStatistics().
+            typedef math::ISOpMagnitude<math::ISGradient<math::FD_1ST> > MathOp;
+            ex = tools::opExtrema(grid.cbeginValueOn(), MathOp(), threaded);
+            CPPUNIT_ASSERT_DOUBLES_EQUAL(double(0.0), ex.min(), /*tolerance=*/0.0);
+            CPPUNIT_ASSERT_DOUBLES_EQUAL(
+                double(9.0 + 9.0 + 9.0), ex.max() * ex.max(), /*tolerance=*/1.0e-3);
+                // max gradient is (dx, dy, dz) = (-3 - 0, -3 - 0, -3 - 0)
+        }
+    }
+    {
+        const Vec3s background(0.0);
+        Vec3SGrid grid(background);
+
+        // Compute active vector magnitude statistics for a vector-valued grid
+        // with two active voxel populations of the same size but two different values.
+        grid.fill(CoordBBox::createCube(Coord(0), DIM),    Vec3s(3.0, 0.0, 4.0)); // length = 5
+        grid.fill(CoordBBox::createCube(Coord(-300), DIM), Vec3s(1.0, 2.0, 2.0)); // length = 3
+
+        CPPUNIT_ASSERT_EQUAL(Index64(2 * DIM * DIM * DIM), grid.activeVoxelCount());
+
+        for (int threaded = 0; threaded <= 1; ++threaded) {
+            math::Extrema ex = tools::extrema(grid.cbeginValueOn(), threaded);
+
+            CPPUNIT_ASSERT_DOUBLES_EQUAL(double(3.0), ex.min(),  /*tolerance=*/0.0);
+            CPPUNIT_ASSERT_DOUBLES_EQUAL(double(5.0), ex.max(),  /*tolerance=*/0.0);
+        }
+    }
+}
+
 void
 TestStats::testGridStats()
 {
@@ -475,7 +658,7 @@ TestStats::testGridHistogram()
             math::Histogram hist = tools::histogram(grid.cbeginValueOn(),
                 /*min=*/0.0, /*max=*/100.0);
 
-            for (size_t i = 0, N = hist.numBins(); i < N; ++i) {
+            for (int i = 0, N = int(hist.numBins()); i < N; ++i) {
                 uint64_t expected = ((hist.min(i) <= value && value <= hist.max(i)) ? 1 : 0);
                 CPPUNIT_ASSERT_EQUAL(expected, hist.count(i));
             }
@@ -493,7 +676,7 @@ TestStats::testGridHistogram()
                 /*min=*/0.0, /*max=*/10.0, /*numBins=*/9, threaded);
 
             CPPUNIT_ASSERT_EQUAL(Index64(2 * DIM * DIM * DIM), hist.size());
-            for (size_t i = 0, N = hist.numBins(); i < N; ++i) {
+            for (int i = 0, N = int(hist.numBins()); i < N; ++i) {
                 if (i == 0 || i == 2) {
                     CPPUNIT_ASSERT_EQUAL(uint64_t(DIM * DIM * DIM), hist.count(i));
                 } else {
@@ -519,7 +702,7 @@ TestStats::testGridHistogram()
                 /*min=*/0.0, /*max=*/10.0, /*numBins=*/9, threaded);
 
             CPPUNIT_ASSERT_EQUAL(Index64(2 * DIM * DIM * DIM), hist.size());
-            for (size_t i = 0, N = hist.numBins(); i < N; ++i) {
+            for (int i = 0, N = int(hist.numBins()); i < N; ++i) {
                 if (i == 2 || i == 4) {
                     CPPUNIT_ASSERT_EQUAL(uint64_t(DIM * DIM * DIM), hist.count(i));
                 } else {
@@ -530,6 +713,6 @@ TestStats::testGridHistogram()
     }
 }
 
-// Copyright (c) 2012-2013 DreamWorks Animation LLC
+// Copyright (c) 2012-2014 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
