@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2012-2013 DreamWorks Animation LLC
+// Copyright (c) 2012-2014 DreamWorks Animation LLC
 //
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
@@ -45,12 +45,14 @@ public:
     CPPUNIT_TEST(testTransformEquality);
     CPPUNIT_TEST(testBackwardCompatibility);
     CPPUNIT_TEST(testIsIdentity);
+    CPPUNIT_TEST(testBoundingBoxes);
     CPPUNIT_TEST_SUITE_END();
 
     void testLinearTransform();
     void testTransformEquality();
     void testBackwardCompatibility();
     void testIsIdentity();
+    void testBoundingBoxes();
 };
 
 CPPUNIT_TEST_SUITE_REGISTRATION(TestTransform);
@@ -95,7 +97,7 @@ TestTransform::testLinearTransform()
     CPPUNIT_ASSERT_DOUBLES_EQUAL(0.5, voxelSize[0], TOL);
     CPPUNIT_ASSERT_DOUBLES_EQUAL(0.5, voxelSize[1], TOL);
     CPPUNIT_ASSERT_DOUBLES_EQUAL(0.5, voxelSize[2], TOL);
-    
+
     CPPUNIT_ASSERT(t->hasUniformScale());
 
     // world to index space
@@ -156,10 +158,10 @@ TestTransform::testLinearTransform()
     // with diagonal -2, 2, -2
     const double PI = std::atan(1.0)*4;
     t->preRotate(PI, math::Y_AXIS);
-    
+
     // this is just a rotation so it will have uniform scale
     CPPUNIT_ASSERT(t->hasUniformScale());
- 
+
     CPPUNIT_ASSERT_EQUAL(math::ScaleMap::mapType(), t->baseMap()->type());
 
     voxelSize = t->voxelSize();
@@ -277,18 +279,18 @@ void
 TestTransform::testTransformEquality()
 {
     using namespace openvdb;
-    
+
     // maps created in different ways may be equivalent
     math::Transform::Ptr t1 = math::Transform::createLinearTransform(0.5);
     math::Mat4d mat = math::Mat4d::identity();
     mat.preScale(math::Vec3d(0.5, 0.5, 0.5));
     math::Transform::Ptr t2 = math::Transform::createLinearTransform(mat);
-    
+
     CPPUNIT_ASSERT( *t1 == *t2);
 
     // test that the auto-convert to the simplest form worked
     CPPUNIT_ASSERT( t1->mapType() == t2->mapType());
-    
+
 
     mat.preScale(math::Vec3d(1., 1., .4));
     math::Transform::Ptr t3 = math::Transform::createLinearTransform(mat);
@@ -296,18 +298,20 @@ TestTransform::testTransformEquality()
     CPPUNIT_ASSERT( *t1 != *t3);
 
     // test equality between different but equivalent maps
-    math::UniformScaleTranslateMap::Ptr ustmap( new math::UniformScaleTranslateMap(1.0, math::Vec3d(0,0,0)));
+    math::UniformScaleTranslateMap::Ptr ustmap(
+        new math::UniformScaleTranslateMap(1.0, math::Vec3d(0,0,0)));
     math::Transform::Ptr t4( new math::Transform( ustmap) );
     CPPUNIT_ASSERT( t4->baseMap()->isType<math::UniformScaleMap>() );
     math::Transform::Ptr t5( new math::Transform);  // constructs with a scale map
     CPPUNIT_ASSERT( t5->baseMap()->isType<math::ScaleMap>() );
 
     CPPUNIT_ASSERT( *t5 == *t4);
-      
+
     CPPUNIT_ASSERT( t5->mapType() != t4->mapType() );
-    
+
     // test inequatlity of two maps of the same type
-    math::UniformScaleTranslateMap::Ptr ustmap2( new math::UniformScaleTranslateMap(1.0, math::Vec3d(1,0,0)));
+    math::UniformScaleTranslateMap::Ptr ustmap2(
+        new math::UniformScaleTranslateMap(1.0, math::Vec3d(1,0,0)));
     math::Transform::Ptr t6( new math::Transform( ustmap2) );
     CPPUNIT_ASSERT( t6->baseMap()->isType<math::UniformScaleTranslateMap>() );
     CPPUNIT_ASSERT( *t6 != *t4);
@@ -317,7 +321,7 @@ TestTransform::testTransformEquality()
     math::Transform::Ptr frustum = math::Transform::createFrustumTransform(bbox, 0.25, 10);
 
     CPPUNIT_ASSERT( *frustum != *t1 );
-        
+
 
 }
 ////////////////////////////////////////
@@ -457,7 +461,7 @@ TestTransform::testBackwardCompatibility()
     t->read(ss);
 
     CPPUNIT_ASSERT_EQUAL(math::AffineMap::mapType(), t->baseMap()->type());
-    
+
     voxelSize = t->voxelSize();
 
     CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, voxelSize[0], TOL);
@@ -477,16 +481,16 @@ TestTransform::testIsIdentity()
 {
     using namespace openvdb;
     math::Transform::Ptr t = math::Transform::createLinearTransform(1.0);
-    
+
     CPPUNIT_ASSERT(t->isIdentity());
-    
+
     t->preScale(Vec3d(2,2,2));
-    
+
     CPPUNIT_ASSERT(!t->isIdentity());
-    
+
     t->preScale(Vec3d(0.5,0.5,0.5));
     CPPUNIT_ASSERT(t->isIdentity());
-    
+
     BBoxd bbox(math::Vec3d(-5,-5,0), Vec3d(5,5,10));
     math::Transform::Ptr f = math::Transform::createFrustumTransform(bbox,
                                                                      /*taper*/ 1,
@@ -504,6 +508,46 @@ TestTransform::testIsIdentity()
     CPPUNIT_ASSERT(f->isIdentity());
 }
 
+
+void
+TestTransform::testBoundingBoxes()
+{
+    using namespace openvdb;
+
+    {
+        math::Transform::ConstPtr t = math::Transform::createLinearTransform(0.5);
+
+        const BBoxd bbox(Vec3d(-8.0), Vec3d(16.0));
+
+        BBoxd xBBox = t->indexToWorld(bbox);
+        CPPUNIT_ASSERT_EQUAL(Vec3d(-4.0), xBBox.min());
+        CPPUNIT_ASSERT_EQUAL(Vec3d(8.0), xBBox.max());
+
+        xBBox = t->worldToIndex(xBBox);
+        CPPUNIT_ASSERT_EQUAL(bbox.min(), xBBox.min());
+        CPPUNIT_ASSERT_EQUAL(bbox.max(), xBBox.max());
+    }
+    {
+        const double PI = std::atan(1.0) * 4.0, SQRT2 = std::sqrt(2.0);
+
+        math::Transform::Ptr t = math::Transform::createLinearTransform(1.0);
+        t->preRotate(PI / 4.0, math::Z_AXIS);
+
+        const BBoxd bbox(Vec3d(-10.0), Vec3d(10.0));
+
+        BBoxd xBBox = t->indexToWorld(bbox); // expand in x and y by sqrt(2)
+        CPPUNIT_ASSERT(Vec3d(-10.0 * SQRT2, -10.0 * SQRT2, -10.0).eq(xBBox.min()));
+        CPPUNIT_ASSERT(Vec3d(10.0 * SQRT2, 10.0 * SQRT2, 10.0).eq(xBBox.max()));
+
+        xBBox = t->worldToIndex(xBBox); // expand again in x and y by sqrt(2)
+        CPPUNIT_ASSERT(Vec3d(-20.0, -20.0, -10.0).eq(xBBox.min()));
+        CPPUNIT_ASSERT(Vec3d(20.0, 20.0, 10.0).eq(xBBox.max()));
+    }
+
+    /// @todo frustum transform
+}
+
+
 ////////////////////////////////////////
 
 
@@ -517,6 +561,6 @@ TestTransform::testNonlinearTransform()
 }
 */
 
-// Copyright (c) 2012-2013 DreamWorks Animation LLC
+// Copyright (c) 2012-2014 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
