@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2012-2013 DreamWorks Animation LLC
+// Copyright (c) 2012-2014 DreamWorks Animation LLC
 //
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
@@ -32,23 +32,27 @@
 #define OPENVDB_VERSION_HAS_BEEN_INCLUDED
 
 #include "Platform.h"
-#include <iosfwd> // for std::istream
-#include <string>
 
 
 /// The version namespace name for this library version
 ///
 /// Fully-namespace-qualified symbols are named as follows:
-/// vdb::vX_Y_Z::Vec3i, vdb::vX_Y_Z::io::File, vdb::vX_Y_Z::tree::Tree, etc.,
+/// openvdb::vX_Y_Z::Vec3i, openvdb::vX_Y_Z::io::File, openvdb::vX_Y_Z::tree::Tree, etc.,
 /// where X, Y and Z are OPENVDB_LIBRARY_MAJOR_VERSION, OPENVDB_LIBRARY_MINOR_VERSION
 /// and OPENVDB_LIBRARY_PATCH_VERSION, respectively (defined below).
-#define OPENVDB_VERSION_NAME v2_3_0
+#define OPENVDB_VERSION_NAME v3_0_0
 
 // Library major, minor and patch version numbers
-#define OPENVDB_LIBRARY_MAJOR_VERSION_NUMBER 2
-#define OPENVDB_LIBRARY_MINOR_VERSION_NUMBER 3
+#define OPENVDB_LIBRARY_MAJOR_VERSION_NUMBER 3
+#define OPENVDB_LIBRARY_MINOR_VERSION_NUMBER 0
 #define OPENVDB_LIBRARY_PATCH_VERSION_NUMBER 0
 
+/// @brief Library version number string of the form "<major>.<minor>.<patch>"
+/// @details This is a macro rather than a static constant because we typically
+/// want the compile-time version number, not the runtime version number
+/// (although the two are usually the same).
+#define OPENVDB_LIBRARY_VERSION_STRING "3.0.0"
+
 /// Library version number as a packed integer ("%02x%02x%04x", major, minor, patch)
 #define OPENVDB_LIBRARY_VERSION_NUMBER \
     ((OPENVDB_LIBRARY_MAJOR_VERSION_NUMBER << 24) | \
@@ -56,8 +60,8 @@
     (OPENVDB_LIBRARY_PATCH_VERSION_NUMBER & 0xFFFF))
 
 /// If OPENVDB_REQUIRE_VERSION_NAME is undefined, symbols from the version
-/// namespace are promoted to the top-level namespace (e.g., vdb::v1_0_0::io::File
-/// can be referred to simply as vdb::io::File).  Otherwise, symbols must be fully
+/// namespace are promoted to the top-level namespace (e.g., openvdb::v1_0_0::io::File
+/// can be referred to simply as openvdb::io::File).  Otherwise, symbols must be fully
 /// namespace-qualified.
 #ifdef OPENVDB_REQUIRE_VERSION_NAME
 #define OPENVDB_USE_VERSION_NAMESPACE
@@ -89,7 +93,7 @@ const uint32_t OPENVDB_LIBRARY_VERSION = OPENVDB_LIBRARY_VERSION_NUMBER;
 /// @brief The current version number of the VDB file format
 /// @details  This can be used to enable various backwards compatability switches
 /// or to reject files that cannot be read.
-const uint32_t OPENVDB_FILE_VERSION = 222;
+const uint32_t OPENVDB_FILE_VERSION = 223;
 
 /// Notable file format version numbers
 enum {
@@ -103,51 +107,27 @@ enum {
     OPENVDB_FILE_VERSION_NEW_TRANSFORM = 219,
     OPENVDB_FILE_VERSION_SELECTIVE_COMPRESSION = 220,
     OPENVDB_FILE_VERSION_FLOAT_FRUSTUM_BBOX = 221,
-    OPENVDB_FILE_VERSION_NODE_MASK_COMPRESSION = 222
+    OPENVDB_FILE_VERSION_NODE_MASK_COMPRESSION = 222,
+    OPENVDB_FILE_VERSION_BLOSC_COMPRESSION = 223,
+    OPENVDB_FILE_VERSION_POINT_INDEX_GRID = 223
 };
 
 
-struct VersionId { uint32_t first, second; VersionId(): first(0), second(0) {} };
+/// Return a library version number string of the form "<major>.<minor>.<patch>".
+inline const char* getLibraryVersionString() { return OPENVDB_LIBRARY_VERSION_STRING; }
 
-namespace io {
-/// @brief Return the file format version number associated with the given input stream.
-OPENVDB_API uint32_t getFormatVersion(std::istream&);
-/// @brief Return the (major, minor) library version number associated with the given input stream.
-OPENVDB_API VersionId getLibraryVersion(std::istream&);
-/// @brief Return a string of the form "<major>.<minor>/<format>", giving the library
-/// and file format version numbers associated with the given input stream.
-OPENVDB_API std::string getVersion(std::istream&);
-// Associate the current file format and library version numbers with the given input stream.
-OPENVDB_API void setCurrentVersion(std::istream&);
-// Associate specific file format and library version numbers with the given stream.
-OPENVDB_API void setVersion(std::ios_base&, const VersionId& libraryVersion, uint32_t fileVersion);
-// Return a bitwise OR of compression option flags (COMPRESS_ZIP, COMPRESS_ACTIVE_MASK, etc.)
-// specifying whether and how input data is compressed or output data should be compressed.
-OPENVDB_API uint32_t getDataCompression(std::ios_base&);
-// Associate with the given stream a bitwise OR of compression option flags (COMPRESS_ZIP,
-// COMPRESS_ACTIVE_MASK, etc.) specifying whether and how input data is compressed
-// or output data should be compressed.
-OPENVDB_API void setDataCompression(std::ios_base&, uint32_t compressionFlags);
-// Return the class (GRID_LEVEL_SET, GRID_UNKNOWN, etc.) of the grid
-// currently being read from or written to the given stream.
-OPENVDB_API uint32_t getGridClass(std::ios_base&);
-// brief Associate with the given stream the class (GRID_LEVEL_SET, GRID_UNKNOWN, etc.)
-// of the grid currently being read or written.
-OPENVDB_API void setGridClass(std::ios_base&, uint32_t);
-// Return a pointer to the background value of the grid currently being
-// read from or written to the given stream.
-OPENVDB_API const void* getGridBackgroundValuePtr(std::ios_base&);
-// Specify (a pointer to) the background value of the grid currently being
-// read from or written to the given stream.
-// The pointer must remain valid until the entire grid has been read or written.
-OPENVDB_API void setGridBackgroundValuePtr(std::ios_base&, const void* background);
-} // namespace io
+
+struct VersionId {
+    uint32_t first, second;
+    VersionId(): first(0), second(0) {}
+    VersionId(uint32_t major, uint32_t minor): first(major), second(minor) {}
+};
 
 } // namespace OPENVDB_VERSION_NAME
 } // namespace openvdb
 
 #endif // OPENVDB_VERSION_HAS_BEEN_INCLUDED
 
-// Copyright (c) 2012-2013 DreamWorks Animation LLC
+// Copyright (c) 2012-2014 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
